//===========================================================================
//
//	HomeWork from Belgium			Not licensed software  
//	1999 - 2000						No rights reserved
//
//===========================================================================
//
//	Project/Product :	Iconizer
//  FileName		:	HookList.cpp
//	Author(s)		:	Bart Gysens
//
//	Description		:	Implementation file for the CHookList class
//
//	Classes			:	CHookList
//
//	Information		:
//	  Compiler(s)	:	Visual C++ 6.0 SP4
//	  Target(s)		:	Windows NT and Windows 2000
//	  Editor		:	Visual C++ 6.0
//
//	History
//	Vers.  Date      Aut.  Type     Description
//  -----  --------  ----  -------  -----------------------------------------
//	1.00   06 09 00  BG    Create   Original
//
//===========================================================================

#include "stdafx.h"
#include "malloc.h"
#include "assert.h"
#include "trayicon.h"
#include "hooklist.h"

//===========================================================================
//	Externals
//===========================================================================

extern LONG DoSetWindowLong( HWND hWnd, int nIndex, LONG dwNewLong );
extern CTrayIcon g_TrayIcon;


//===========================================================================
//	Macros and typedefs
//===========================================================================


//===========================================================================
// 
//	Class			:	CHookList
//	Author(s)		:	Bart Gysens
//
//	Description		:	Declaration of the CHookList class
//
//	Comments		:	None
//
//	History			:	1.00  : Create
//
//===========================================================================

CHookList::CHookList()
{
	m_NbrItems  = 0;
	m_pHookItem = NULL;
}

CHookList::~CHookList()
{
	CleanUp();
}

void CHookList::AddItem( CHookItem* pItem )
{
	m_NbrItems++;
	if ( m_pHookItem == NULL )
		m_pHookItem = (CHookItem*) malloc( m_NbrItems * sizeof( CHookItem ) );
	else
		m_pHookItem = (CHookItem*) realloc( m_pHookItem, m_NbrItems * sizeof( CHookItem ) );
	memcpy( m_pHookItem+(m_NbrItems-1), pItem, sizeof( CHookItem ) ); 
}

CHookItem * CHookList::GetItem( HWND hWnd )
{
	CHookItem * pItem = m_pHookItem;
	for( UINT i=0; i<m_NbrItems && pItem->m_hWnd!=hWnd; i++, pItem++ );	
	return ( i < m_NbrItems ) ? pItem : NULL;
}

void CHookList::DelItem( HWND hWnd )
{
	CHookItem * pItem = m_pHookItem;
	for( UINT i=0; i<m_NbrItems && pItem->m_hWnd!=hWnd; i++, pItem++ );
	if ( i < m_NbrItems )
	{
		m_NbrItems--;
		memcpy( pItem, pItem+1, ( m_NbrItems - i ) * sizeof( CHookItem ) );
		m_pHookItem = (CHookItem*) realloc( m_pHookItem, m_NbrItems * sizeof( CHookItem ) ); 
	}
}

void CHookList::DelAllItems()
{
	if ( m_pHookItem != NULL )
		free( m_pHookItem );
	m_pHookItem = NULL;
	m_NbrItems  = 0;
}

void CHookList::UnhookAllItems()
{
	CHookItem * pItem;

	while ( m_NbrItems > 0 )
	{
		pItem = m_pHookItem+(m_NbrItems-1);

		// Unhook window
		HWND hWnd = pItem->m_hWnd;
		DoSetWindowLong( hWnd, GWL_WNDPROC, (LONG) pItem->m_DefWndProc );			
		if ( !pItem->m_bIconized )
			SendMessage( hWnd, WM_NCPAINT, 1, 0 );
		else
			{
			g_TrayIcon.DelTrayIcon( hWnd );
			ShowWindow( hWnd, SW_SHOW );
			SendMessage( hWnd, WM_SYSCOMMAND, SC_MINIMIZE, 0 );
			}

		// Delete item
		DelItem( hWnd );
	}
}

void CHookList::CleanUp()
{
	UnhookAllItems();
	DelAllItems();
}