//===========================================================================
//
//	HomeWork from Belgium			Not licensed software  
//	1999 - 2000						No rights reserved
//
//===========================================================================
//
//	Project/Product :	Iconizer DLL
//  FileName		:	Iconizer DLL.cpp
//	Author(s)		:	Bart Gysens
//
//	Description		:	Implementation file of the Iconize DLL
//
//	Classes			:	
//
//	Information		:
//	  Compiler(s)	:	Visual C++ 6.0 SP4
//	  Target(s)		:	Windows NT and Windows 2000
//	  Editor		:	Visual C++ 6.0 internal editor
//
//	History
//	Vers.  Date      Aut.  Type     Description
//  -----  --------  ----  -------  -----------------------------------------
//	1.00   06 09 00  BG    Create   Original
//
//===========================================================================

#include "stdafx.h"
#include "stdio.h"
#include "stdlib.h"
#define DLLAPI_ICONIZER __declspec( dllexport )
#include "iconizer DLL.h"
#include "hooklist.h"
#include "caption.h"
#include "trayicon.h"


//===========================================================================
//	Macros and typedefs
//===========================================================================

#define MSG_HOOK			( TEXT( "MSG_HOOK" ) )
#define	MSG_UNHOOK			( TEXT( "MSG_UNHOOK" ) )
#define MSG_TRAY			( TEXT( "MSG_TRAY" ) )
#define WS_TEST_STYLES		( WS_CAPTION | WS_MINIMIZEBOX | WS_SYSMENU )
#define GETDEFWNDPROC( h )	( (WNDPROC) ( IsWindowUnicode( h ) ? GetWindowLongW( h, GWL_WNDPROC ) : GetWindowLongA( h, GWL_WNDPROC ) ) )


//===========================================================================
//	Global variables
//===========================================================================

#pragma data_seg( "ICONIZER" )
HINSTANCE	g_hModule	= NULL	;
HHOOK		g_hHook		= NULL	;
CTrayIcon   g_TrayIcon			;
UINT		g_WmHook    = 0		;
UINT		g_WmUnHook  = 0		;
UINT		g_WmTray	= 0		;
CCaption	g_Caption			;
CHookList	g_HookList			;
#pragma data_seg()
#pragma comment( linker, "/SECTION:ICONIZER,rws" ) // read, write and share


//===========================================================================
// 
//	Function		:	DoCallWindowProc
//	Author(s)		:	Bart Gysens
//
//	Description		:	Determines which window procedure to call,
//						i.e. ANSI or UNICODE
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

void Log( char * pFmt, ... )
{
	FILE	*	pFile		;
	char	*	pBuf		;
	char		Path[512]	;

	if ( ( pBuf = getenv( "TMP" ) ) == NULL )
		pBuf = getenv( "TEMP" );

	if ( pBuf != NULL )
	{
		strcpy( Path, pBuf );
		if ( Path[strlen( Path ) - 1] != '\\' )
			strcat( Path, "\\" );
		strcat( Path, "Iconizer.log" );

		if ( ( pFile = fopen( Path, "at" ) ) != NULL )
		{
			va_list	arg		;
			va_start( arg, pFmt );
			vfprintf( pFile, pFmt, arg );
			va_end( arg );

			if ( pFmt[strlen( pFmt ) - 1] != '\n' )
				fprintf( pFile, "\n" );
			fclose( pFile );
		}
	}
}


//===========================================================================
// 
//	Function		:	DoCallWindowProc
//	Author(s)		:	Bart Gysens
//
//	Description		:	Determines which window procedure to call,
//						i.e. ANSI or UNICODE
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

LRESULT DoCallWindowProc( WNDPROC lpPrevWndFunc, HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam )
{
	LRESULT RetVal;
	if ( ::IsWindowUnicode( hWnd ) )
		RetVal = CallWindowProcW( lpPrevWndFunc, hWnd, Msg, wParam, lParam );
	else
		RetVal = CallWindowProcA( lpPrevWndFunc, hWnd, Msg, wParam, lParam );
	return RetVal;
}


//===========================================================================
// 
//	Function		:	DoSetWindowLong
//	Author(s)		:	Bart Gysens
//
//	Description		:	Determines which version of SetWindowLong to call,
//						i.e. ANSI or UNICODE
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

LONG DoSetWindowLong( HWND hWnd, int nIndex, LONG dwNewLong )
{
	LONG RetVal;
	if ( ::IsWindowUnicode( hWnd ) )	
		RetVal = SetWindowLongW( hWnd, nIndex, dwNewLong );
	else
		RetVal = SetWindowLongA( hWnd, nIndex, dwNewLong );
	return RetVal;
}

//===========================================================================
// 
//	Function		:	DoGetWindowLong
//	Author(s)		:	Bart Gysens
//
//	Description		:	Determines which version of GetWindowLong to call,
//						i.e. ANSI or UNICODE
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

LONG DoGetWindowLong( HWND hWnd, int nIndex )
{
	LONG RetVal;
	if ( ::IsWindowUnicode( hWnd ) )	
		RetVal = GetWindowLongW( hWnd, nIndex );
	else
		RetVal = GetWindowLongA( hWnd, nIndex );
	return RetVal;
}


//===========================================================================
// 
//	Function		:	HookProc
//	Author(s)		:	Bart Gysens
//
//	Description		:	Function that implements the hook
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

LRESULT CALLBACK HookWndProc( HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam )
{
	CHookItem * pItem		;
	LRESULT		result = 0	;
	
	// Search for a known window, no processing if unknown
	if ( ( pItem = g_HookList.GetItem( hWnd ) ) != NULL )
	{
		// Hook following messages
		switch( Msg )
		{
			case WM_NCDESTROY:
				DoSetWindowLong( hWnd, GWL_WNDPROC, (LONG) pItem->m_DefWndProc );
				result = DoCallWindowProc( pItem->m_DefWndProc, hWnd, Msg, wParam, lParam );			
				g_HookList.DelItem( hWnd );
				break;

			case WM_NCACTIVATE:
				result = g_Caption.OnNcActivate( hWnd, Msg, wParam, lParam, pItem );
				break;

			case WM_NCLBUTTONDOWN:
				result = g_Caption.OnNcLButtonDown( hWnd, Msg, wParam, lParam, pItem );
				break;

			case WM_NCLBUTTONUP:
				result = g_Caption.OnNcLButtonUp( hWnd, Msg, wParam, lParam, pItem );
				break;

			case WM_NCMOUSEMOVE:
				result = g_Caption.OnNcMouseMove( hWnd, Msg, wParam, lParam, pItem );
				break;

			default:
				if ( Msg == g_WmTray )
					g_TrayIcon.OnNotifyIcon( hWnd, wParam, lParam, pItem );
				else
					result = g_Caption.OnDefault( hWnd, Msg, wParam, lParam, pItem );
				break;
		}
	}

	return result;
}

//===========================================================================
// 
//	Function		:	IsExcluded
//	Author(s)		:	Bart Gysens
//
//	Description		:	Check's if a window is excluded for hooking
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

BOOL IsExcluded( HWND hWnd )
{
	const	BufLen			= 256	;
	char	Buf[BufLen*2]			;
	char	Title[BufLen*2]			;
	char	Entry[BufLen]			;
	char *	pTmp					;	
	DWORD	Index			= 0		;
	BOOL	Found			= FALSE	;

	if ( !GetWindowText( hWnd, Title, BufLen ) )
		Found = TRUE;
	else
	{
		while ( 1 )
		{
			sprintf( Entry, "Name %i", ++Index );
			GetPrivateProfileString( "Exclude List", Entry, "", Buf, BufLen, "Iconizer.ini" );
			if ( Buf[0] == '\0' )
				break;
			else
				{
				strupr( Buf );
				strupr( Title );

				if ( ( pTmp = strstr( Buf, "*" ) ) != NULL )
				{
					pTmp[0] = '\0';
					if ( strstr( Title, Buf ) != NULL )
					{
						Found = TRUE;
						break;
					}
				}
				else if ( !strcmp( Title, Buf ) )
				{
					Found = TRUE;
					break;
				}				
			}		
		}
	}

	return Found;
}


//===========================================================================
// 
//	Function		:	HookProc
//	Author(s)		:	Bart Gysens
//
//	Description		:	Function that will replace the normal WndProc with
//						the HookWndProc procedure
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

LRESULT WINAPI CallWndHookProc( int nCode, WPARAM wParam, LPARAM lParam )
{
	if ( nCode == HC_ACTION )
	{
		LPCWPSTRUCT pCwp	  = (LPCWPSTRUCT) lParam;
		DWORD		dwStyle	  = DoGetWindowLong( pCwp->hwnd, GWL_STYLE );
		DWORD		dwExStyle = DoGetWindowLong( pCwp->hwnd, GWL_EXSTYLE );

		// Test for valid window to hook
		if (   ( ( dwExStyle & WS_EX_WINDOWEDGE ) == WS_EX_WINDOWEDGE )
			&& ( ( dwStyle & WS_TEST_STYLES ) == WS_TEST_STYLES )
			&& ( g_HookList.GetItem( pCwp->hwnd ) == NULL )
			&& ( GetParent( pCwp->hwnd ) == NULL ) )
		{
			if (   (   ( pCwp->message == g_WmHook )
				    || ( pCwp->message == WM_NCACTIVATE ) )
				&& ( !IsExcluded( pCwp->hwnd ) ) )
			{
				g_HookList.AddItem( &CHookItem( pCwp->hwnd, HookWndProc, GETDEFWNDPROC( pCwp->hwnd ) ) );
				DoSetWindowLong( pCwp->hwnd, GWL_WNDPROC, (LONG) HookWndProc );
			}
		}
	}
	
	return CallNextHookEx( g_hHook, nCode, wParam, lParam );
}


//===========================================================================
// 
//	Function		:	EnumProc
//	Author(s)		:	Bart Gysens
//
//	Description		:	Hook/Unhook all top level windows
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

BOOL CALLBACK EnumProc( HWND hWnd, LPARAM lParam )
{
	if ( lParam == 0 )
		SendMessage( hWnd, g_WmHook, 0, 0 );
	else
		SendMessage( hWnd, g_WmUnHook, 0, 0 );
	return TRUE;
}


//===========================================================================
// 
//	Function		:	StartHook
//	Author(s)		:	Bart Gysens
//
//	Description		:	Function to start system wide hook
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

BOOL WINAPI StartHook( void )
{
	if ( g_hHook == NULL )
	{
		g_hHook = SetWindowsHookEx( WH_CALLWNDPROC, CallWndHookProc, g_hModule, 0 );
		EnumWindows( EnumProc, 0 );
		return TRUE;
	}
	return FALSE;
}

//===========================================================================
// 
//	Function		:	StopHook
//	Author(s)		:	Bart Gysens
//
//	Description		:	Function to stop system wide hook
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

BOOL WINAPI StopHook( void )
{
	if ( g_hHook != NULL )
	{
		g_HookList.CleanUp();
		EnumWindows( EnumProc, 1 );
		if ( UnhookWindowsHookEx( g_hHook ) != 0 )
			g_hHook = NULL;				
	}

	return( g_hHook == NULL );
}

//===========================================================================
// 
//	Function		:	DllMain
//	Author(s)		:	Bart Gysens
//
//	Description		:	Implementation of the main dll procedure
//
//	Comments		:	none
//
//	History			:	1.00  : Create
//
//===========================================================================

BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved )
{
    switch ( ul_reason_for_call )
	{
		case DLL_PROCESS_ATTACH:
			// Save module handle
			g_hModule = (HINSTANCE) hModule;

			// 	Register messages
			if ( g_WmHook == 0 )
				g_WmHook = RegisterWindowMessage( MSG_HOOK );
			if ( g_WmUnHook == 0 )
				g_WmUnHook = RegisterWindowMessage( MSG_UNHOOK );;
			if ( g_WmTray == 0 )
				g_WmTray = RegisterWindowMessage( MSG_TRAY );
			break;

		case DLL_PROCESS_DETACH:
			g_HookList.CleanUp();
			break;
    }

    return TRUE;
}
